@@ -15,9 +15,9 @@ var bench = common.createBenchmark(main, {
 const INT8   = 0x7f;
 const INT16  = 0x7fff;
 const INT32  = 0x7fffffff;
-const UINT8  = INT8 * 2;
-const UINT16 = INT16 * 2;
-const UINT32 = INT32 * 2;
+const UINT8  = (INT8 * 2) + 1;
+const UINT16 = (INT16 * 2) + 1;
+const UINT32 = INT32;
 
 var mod = {
   writeInt8: INT8,
@@ -47,17 +47,23 @@ function main(conf) {
 
 function benchInt(buff, fn, len, noAssert) {
   var m = mod[fn];
+  var testFunction = new Function('buff', [
+    "for (var i = 0; i !== " + len + "; i++) {",
+    "  buff." + fn + "(i & " + m + ", 0, " + JSON.stringify(noAssert) + ");",
+    "}"
+  ].join("\n"));
   bench.start();
-  for (var i = 0; i < len; i++) {
-    buff[fn](i % m, 0, noAssert);
-  }
+  testFunction(buff);
   bench.end(len / 1e6);
 }
 
 function benchFloat(buff, fn, len, noAssert) {
+  var testFunction = new Function('buff', [
+    "for (var i = 0; i !== " + len + "; i++) {",
+    "  buff." + fn + "(i, 0, " + JSON.stringify(noAssert) + ");",
+    "}"
+  ].join("\n"));
   bench.start();
-  for (var i = 0; i < len; i++) {
-    buff[fn](i * 0.1, 0, noAssert);
-  }
+  testFunction(buff);
   bench.end(len / 1e6);
 }
