@@ -1,37 +1,23 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
-var util = require('util'),
-    path = require('path'),
-    net = require('net'),
-    vm = require('vm'),
-    repl = require('repl'),
-    inherits = util.inherits,
-    spawn = require('child_process').spawn;
+const util = require('util');
+const path = require('path');
+const net = require('net');
+const vm = require('vm');
+const Module = require('module');
+const repl = require('repl');
+const inherits = util.inherits;
+const assert = require('assert');
+const spawn = require('child_process').spawn;
+const Buffer = require('buffer').Buffer;
 
 exports.start = function(argv, stdin, stdout) {
   argv || (argv = process.argv.slice(2));
 
   if (argv.length < 1) {
     console.error('Usage: node debug script.js');
+    console.error('       node debug <host>:<port>');
+    console.error('       node debug -p <pid>');
     process.exit(1);
   }
 
@@ -59,7 +45,7 @@ exports.port = 5858;
 
 //
 // Parser/Serializer for V8 debugger protocol
-// http://code.google.com/p/v8/wiki/DebuggerProtocol
+// https://github.com/v8/v8/wiki/Debugging-Protocol
 //
 // Usage:
 //    p = new Protocol();
@@ -116,7 +102,7 @@ Protocol.prototype.execute = function(d) {
       if (len - this.bodyStartByteIndex < this.contentLength) {
         break;
       }
-      // pass thru
+      // falls through
     case 'body':
       var resRawByteLength = Buffer.byteLength(res.raw, 'utf8');
 
@@ -140,7 +126,6 @@ Protocol.prototype.execute = function(d) {
 
     default:
       throw new Error('Unknown state');
-      break;
   }
 };
 
@@ -154,7 +139,7 @@ Protocol.prototype.serialize = function(req) {
 };
 
 
-var NO_FRAME = -1;
+const NO_FRAME = -1;
 
 function Client() {
   net.Stream.call(this);
@@ -164,7 +149,6 @@ function Client() {
 
   this.currentFrame = NO_FRAME;
   this.currentSourceLine = -1;
-  this.currentSource = null;
   this.handles = {};
   this.scripts = {};
   this.breakpoints = [];
@@ -182,19 +166,20 @@ exports.Client = Client;
 
 
 Client.prototype._addHandle = function(desc) {
-  if (typeof desc != 'object' || typeof desc.handle != 'number') {
+  if (desc === null || typeof desc !== 'object' ||
+      typeof desc.handle !== 'number') {
     return;
   }
 
   this.handles[desc.handle] = desc;
 
-  if (desc.type == 'script') {
+  if (desc.type === 'script') {
     this._addScript(desc);
   }
 };
 
 
-var natives = process.binding('natives');
+const natives = process.binding('natives');
 
 
 Client.prototype._addScript = function(desc) {
@@ -249,6 +234,10 @@ Client.prototype._onResponse = function(res) {
     this._removeScript(res.body.body.script);
     handled = true;
 
+  } else if (res.body && res.body.event === 'compileError') {
+    // This event is not used anywhere right now, perhaps somewhere in the
+    // future?
+    handled = true;
   }
 
   if (cb) {
@@ -273,7 +262,7 @@ Client.prototype.req = function(req, cb) {
 
 Client.prototype.reqVersion = function(cb) {
   cb = cb || function() {};
-  this.req({ command: 'version' } , function(err, body, res) {
+  this.req({ command: 'version' }, function(err, body, res) {
     if (err) return cb(err);
     cb(null, res.body.body.V8Version, res.body.running);
   });
@@ -296,7 +285,7 @@ Client.prototype.reqLookup = function(refs, cb) {
   this.req(req, function(err, res) {
     if (err) return cb(err);
     for (var ref in res) {
-      if (typeof res[ref] == 'object') {
+      if (res[ref] !== null && typeof res[ref] === 'object') {
         self._addHandle(res[ref]);
       }
     }
@@ -366,7 +355,7 @@ Client.prototype.reqEval = function(expression, cb) {
 };
 
 
-// Finds the first scope in the array in which the epxression evals.
+// Finds the first scope in the array in which the expression evals.
 Client.prototype._reqFramesEval = function(expression, evalFrames, cb) {
   if (evalFrames.length == 0) {
     // Just eval in global scope.
@@ -409,7 +398,7 @@ Client.prototype.reqFrameEval = function(expression, frame, cb) {
 // reqBacktrace(cb)
 // TODO: from, to, bottom
 Client.prototype.reqBacktrace = function(cb) {
-  this.req({ command: 'backtrace', arguments: { inlineRefs: true } } , cb);
+  this.req({ command: 'backtrace', arguments: { inlineRefs: true } }, cb);
 };
 
 
@@ -443,7 +432,7 @@ Client.prototype.reqScripts = function(cb) {
   var self = this;
   cb = cb || function() {};
 
-  this.req({ command: 'scripts' } , function(err, res) {
+  this.req({ command: 'scripts' }, function(err, res) {
     if (err) return cb(err);
 
     for (var i = 0; i < res.length; i++) {
@@ -558,9 +547,7 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
           mirrorValue = '[?]';
         }
 
-
-        if (Array.isArray(mirror) &&
-            typeof prop.name != 'number') {
+        if (Array.isArray(mirror) && typeof prop.name !== 'number') {
           // Skip the 'length' property.
           return;
         }
@@ -600,9 +587,7 @@ Client.prototype.mirrorObject = function(handle, depth, cb) {
   } else {
     val = handle;
   }
-  process.nextTick(function() {
-    cb(null, val);
-  });
+  process.nextTick(cb, null, val);
 };
 
 
@@ -656,11 +641,7 @@ Client.prototype.fullTrace = function(cb) {
 };
 
 
-
-
-
-
-var commands = [
+const commands = [
   [
     'run (r)',
     'cont (c)',
@@ -740,8 +721,7 @@ function SourceInfo(body) {
 // This class is the repl-enabled debugger interface which is invoked on
 // "node debug"
 function Interface(stdin, stdout, args) {
-  var self = this,
-      child;
+  var self = this;
 
   this.stdin = stdin;
   this.stdout = stdout;
@@ -772,6 +752,7 @@ function Interface(stdin, stdout, args) {
   if (parseInt(process.env['NODE_DISABLE_COLORS'], 10)) {
     opts.useColors = false;
   }
+
   this.repl = repl.start(opts);
 
   // Do not print useless warning
@@ -787,22 +768,22 @@ function Interface(stdin, stdout, args) {
   process.once('SIGTERM', process.exit.bind(process, 0));
   process.once('SIGHUP', process.exit.bind(process, 0));
 
-  var proto = Interface.prototype,
-      ignored = ['pause', 'resume', 'exitRepl', 'handleBreak',
-                 'requireConnection', 'killChild', 'trySpawn',
-                 'controlEval', 'debugEval', 'print', 'childPrint',
-                 'clearline'],
-      shortcut = {
-        'run': 'r',
-        'cont': 'c',
-        'next': 'n',
-        'step': 's',
-        'out': 'o',
-        'backtrace': 'bt',
-        'setBreakpoint': 'sb',
-        'clearBreakpoint': 'cb',
-        'pause_': 'pause'
-      };
+  var proto = Interface.prototype;
+  const ignored = ['pause', 'resume', 'exitRepl', 'handleBreak',
+                   'requireConnection', 'killChild', 'trySpawn',
+                   'controlEval', 'debugEval', 'print', 'childPrint',
+                   'clearline'];
+  const shortcut = {
+    'run': 'r',
+    'cont': 'c',
+    'next': 'n',
+    'step': 's',
+    'out': 'o',
+    'backtrace': 'bt',
+    'setBreakpoint': 'sb',
+    'clearBreakpoint': 'cb',
+    'pause_': 'pause'
+  };
 
   function defineProperty(key, protoKey) {
     // Check arity
@@ -857,13 +838,14 @@ function Interface(stdin, stdout, args) {
 
 
 Interface.prototype.pause = function() {
-  if (this.killed || this.paused++ > 0) return false;
+  if (this.killed || this.paused++ > 0) return this;
   this.repl.rli.pause();
   this.stdin.pause();
+  return this;
 };
 
 Interface.prototype.resume = function(silent) {
-  if (this.killed || this.paused === 0 || --this.paused !== 0) return false;
+  if (this.killed || this.paused === 0 || --this.paused !== 0) return this;
   this.repl.rli.resume();
   if (silent !== true) {
     this.repl.displayPrompt();
@@ -874,6 +856,7 @@ Interface.prototype.resume = function(silent) {
     this.waiting();
     this.waiting = null;
   }
+  return this;
 };
 
 
@@ -961,8 +944,8 @@ Interface.prototype.controlEval = function(code, context, filename, callback) {
   try {
     // Repeat last command if empty line are going to be evaluated
     if (this.repl.rli.history && this.repl.rli.history.length > 0) {
-      if (code === '(\n)') {
-        code = '(' + this.repl.rli.history[0] + '\n)';
+      if (code === '\n') {
+        code = this.repl.rli.history[0] + '\n';
       }
     }
 
@@ -1018,28 +1001,13 @@ Interface.prototype.debugEval = function(code, context, filename, callback) {
 
 // Utils
 
-// Returns number of digits (+1)
-function intChars(n) {
-  // TODO dumb:
-  if (n < 50) {
-    return 3;
-  } else if (n < 950) {
-    return 4;
-  } else if (n < 9950) {
-    return 5;
-  } else {
-    return 6;
-  }
-}
-
 // Adds spaces and prefix to number
-function leftPad(n, prefix) {
+// maxN is a maximum number we should have space for
+function leftPad(n, prefix, maxN) {
   var s = n.toString(),
-      nchars = intChars(n),
+      nchars = Math.max(2, String(maxN).length) + 1,
       nspaces = nchars - s.length - 1;
 
-  prefix || (prefix = ' ');
-
   for (var i = 0; i < nspaces; i++) {
     prefix += ' ';
   }
@@ -1138,7 +1106,7 @@ Interface.prototype.list = function(delta) {
       if (lineno == 1) {
         // The first line needs to have the module wrapper filtered out of
         // it.
-        var wrapper = require('module').wrapper[0];
+        var wrapper = Module.wrapper[0];
         lines[i] = lines[i].slice(wrapper.length);
 
         client.currentSourceColumn -= wrapper.length;
@@ -1154,7 +1122,14 @@ Interface.prototype.list = function(delta) {
         line = lines[i];
       }
 
-      self.print(leftPad(lineno, breakpoint && '*') + ' ' + line);
+      var prefixChar = ' ';
+      if (current) {
+        prefixChar = '>';
+      } else if (breakpoint) {
+        prefixChar = '*';
+      }
+
+      self.print(leftPad(lineno, prefixChar, to) + ' ' + line);
     }
     self.resume();
   });
@@ -1215,7 +1190,7 @@ Interface.prototype.scripts = function() {
   this.pause();
   for (var id in client.scripts) {
     var script = client.scripts[id];
-    if (typeof script == 'object' && script.name) {
+    if (script !== null && typeof script === 'object' && script.name) {
       if (displayNatives ||
           script.name == client.currentScript ||
           !script.isNative) {
@@ -1290,11 +1265,11 @@ Interface.prototype.unwatch = function(expr) {
 
 // List watchers
 Interface.prototype.watchers = function() {
-  var self = this,
-      verbose = arguments[0] || false,
-      callback = arguments[1] || function() {},
-      waiting = this._watchers.length,
-      values = [];
+  var self = this;
+  var verbose = arguments[0] || false;
+  var callback = arguments[1] || function() {};
+  var waiting = this._watchers.length;
+  var values = [];
 
   this.pause();
 
@@ -1316,7 +1291,8 @@ Interface.prototype.watchers = function() {
       if (verbose) self.print('Watchers:');
 
       self._watchers.forEach(function(watcher, i) {
-        self.print(leftPad(i, ' ') + ': ' + watcher + ' = ' +
+        self.print(leftPad(i, ' ', self._watchers.length - 1) +
+                   ': ' + watcher + ' = ' +
                    JSON.stringify(values[i]));
       });
 
@@ -1363,6 +1339,12 @@ Interface.prototype.setBreakpoint = function(script, line,
     script = this.client.currentScript;
   }
 
+  if (script === undefined) {
+    this.print('Cannot determine the current script, ' +
+        'make sure the debugged process is paused.');
+    return;
+  }
+
   if (/\(\)$/.test(script)) {
     // setBreakpoint('functionname()');
     var req = {
@@ -1374,7 +1356,7 @@ Interface.prototype.setBreakpoint = function(script, line,
     // setBreakpoint('scriptname')
     if (script != +script && !this.client.scripts[script]) {
       var scripts = this.client.scripts;
-      Object.keys(scripts).forEach(function(id) {
+      for (var id in scripts) {
         if (scripts[id] &&
             scripts[id].name &&
             scripts[id].name.indexOf(script) !== -1) {
@@ -1383,7 +1365,7 @@ Interface.prototype.setBreakpoint = function(script, line,
           }
           scriptId = id;
         }
-      });
+      }
     } else {
       scriptId = script;
     }
@@ -1449,6 +1431,7 @@ Interface.prototype.clearBreakpoint = function(script, line) {
 
   var ambiguous,
       breakpoint,
+      scriptId,
       index;
 
   this.client.breakpoints.some(function(bp, i) {
@@ -1458,6 +1441,7 @@ Interface.prototype.clearBreakpoint = function(script, line) {
       if (index !== undefined) {
         ambiguous = true;
       }
+      scriptId = script;
       if (bp.line === line) {
         index = i;
         breakpoint = bp.id;
@@ -1466,10 +1450,28 @@ Interface.prototype.clearBreakpoint = function(script, line) {
     }
   });
 
+  if (!scriptId && !this.client.scripts[script]) {
+    var scripts = this.client.scripts;
+    for (var id in scripts) {
+      if (scripts[id] &&
+          scripts[id].name &&
+          scripts[id].name.indexOf(script) !== -1) {
+        if (scriptId) {
+          ambiguous = true;
+        }
+        scriptId = id;
+      }
+    }
+  }
+
   if (ambiguous) return this.error('Script name is ambiguous');
 
+  if (scriptId === undefined) {
+    return this.error('Script ' + script + ' not found');
+  }
+
   if (breakpoint === undefined) {
-    return this.error('Script : ' + script + ' not found');
+    return this.error('Breakpoint not found on line ' + line);
   }
 
   var self = this,
@@ -1544,8 +1546,7 @@ Interface.prototype.repl = function() {
   var listeners = this.repl.rli.listeners('SIGINT').slice(0);
   this.repl.rli.removeAllListeners('SIGINT');
 
-  // Exit debug repl on Ctrl + C
-  this.repl.rli.once('SIGINT', function() {
+  function exitDebugRepl() {
     // Restore all listeners
     process.nextTick(function() {
       listeners.forEach(function(listener) {
@@ -1555,7 +1556,16 @@ Interface.prototype.repl = function() {
 
     // Exit debug repl
     self.exitRepl();
-  });
+
+    self.repl.rli.removeListener('SIGINT', exitDebugRepl);
+    self.repl.removeListener('exit', exitDebugRepl);
+  }
+
+  // Exit debug repl on SIGINT
+  this.repl.rli.on('SIGINT', exitDebugRepl);
+
+  // Exit debug repl on repl exit
+  this.repl.on('exit', exitDebugRepl);
 
   // Set new
   this.repl.eval = this.debugEval.bind(this);
@@ -1565,7 +1575,6 @@ Interface.prototype.repl = function() {
   this.history.control = this.repl.rli.history;
   this.repl.rli.history = this.history.debug;
 
-  this.repl.prompt = '> ';
   this.repl.rli.setPrompt('> ');
   this.repl.displayPrompt();
 };
@@ -1581,7 +1590,6 @@ Interface.prototype.exitRepl = function() {
   this.repl.rli.history = this.history.control;
 
   this.repl.context = this.context;
-  this.repl.prompt = 'debug> ';
   this.repl.rli.setPrompt('debug> ');
   this.repl.displayPrompt();
 };
@@ -1616,11 +1624,13 @@ Interface.prototype.trySpawn = function(cb) {
   var self = this,
       breakpoints = this.breakpoints || [],
       port = exports.port,
-      host = 'localhost',
+      host = '127.0.0.1',
       childArgs = this.args;
 
   this.killChild();
+  assert(!this.child);
 
+  var isRemote = false;
   if (this.args.length === 2) {
     var match = this.args[1].match(/^([^:]+):(\d+)$/);
 
@@ -1629,21 +1639,22 @@ Interface.prototype.trySpawn = function(cb) {
       // `node debug localhost:5858`
       host = match[1];
       port = parseInt(match[2], 10);
-      this.child = {
-        kill: function() {
-          // TODO Do we really need to handle it?
-        }
-      };
+      isRemote = true;
     }
   } else if (this.args.length === 3) {
     // `node debug -p pid`
     if (this.args[1] === '-p' && /^\d+$/.test(this.args[2])) {
-      this.child = {
-        kill: function() {
-          // TODO Do we really need to handle it?
+      const pid = parseInt(this.args[2], 10);
+      try {
+        process._debugProcess(pid);
+      } catch (e) {
+        if (e.code === 'ESRCH') {
+          console.error(`Target process: ${pid} doesn't exist.`);
+          process.exit(1);
         }
-      };
-      process._debugProcess(parseInt(this.args[2], 10));
+        throw e;
+      }
+      isRemote = true;
     } else {
       var match = this.args[1].match(/^--port=(\d+)$/);
       if (match) {
@@ -1655,7 +1666,8 @@ Interface.prototype.trySpawn = function(cb) {
     }
   }
 
-  if (!this.child) {
+  if (!isRemote) {
+    // pipe stream into debugger
     this.child = spawn(process.execPath, childArgs);
 
     this.child.stdout.on('data', this.childPrint.bind(this));
@@ -1704,9 +1716,10 @@ Interface.prototype.trySpawn = function(cb) {
 
   client.on('error', connectError);
   function connectError() {
-    // If it's failed to connect 4 times then don't catch the next error
+    // If it's failed to connect 10 times then print failed message
     if (connectionAttempts >= 10) {
-      client.removeListener('error', connectError);
+      console.error(' failed, please retry');
+      process.exit(1);
     }
     setTimeout(attemptConnect, 500);
   }
@@ -1717,8 +1730,12 @@ Interface.prototype.trySpawn = function(cb) {
     client.connect(port, host);
   }
 
-  setTimeout(function() {
-    self.print('connecting..', true);
+  self.print('connecting to ' + host + ':' + port + ' ..', true);
+  if (isRemote) {
     attemptConnect();
-  }, 50);
+  } else {
+    this.child.stderr.once('data', function() {
+      setImmediate(attemptConnect);
+    });
+  }
 };
