@@ -1,25 +1,6 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
-var util = require('util');
+const util = require('util');
 
 function Console(stdout, stderr) {
   if (!(this instanceof Console)) {
@@ -40,13 +21,15 @@ function Console(stdout, stderr) {
   Object.defineProperty(this, '_stdout', prop);
   prop.value = stderr;
   Object.defineProperty(this, '_stderr', prop);
-  prop.value = {};
+  prop.value = new Map();
   Object.defineProperty(this, '_times', prop);
 
   // bind the prototype functions to this Console instance
-  Object.keys(Console.prototype).forEach(function(k) {
+  var keys = Object.keys(Console.prototype);
+  for (var v = 0; v < keys.length; v++) {
+    var k = keys[v];
     this[k] = this[k].bind(this);
-  }, this);
+  }
 }
 
 Console.prototype.log = function() {
@@ -65,18 +48,20 @@ Console.prototype.warn = function() {
 Console.prototype.error = Console.prototype.warn;
 
 
-Console.prototype.dir = function(object) {
-  this._stdout.write(util.inspect(object) + '\n');
+Console.prototype.dir = function(object, options) {
+  this._stdout.write(util.inspect(object, util._extend({
+    customInspect: false
+  }, options)) + '\n');
 };
 
 
 Console.prototype.time = function(label) {
-  this._times[label] = Date.now();
+  this._times.set(label, Date.now());
 };
 
 
 Console.prototype.timeEnd = function(label) {
-  var time = this._times[label];
+  var time = this._times.get(label);
   if (!time) {
     throw new Error('No such label: ' + label);
   }
@@ -85,13 +70,13 @@ Console.prototype.timeEnd = function(label) {
 };
 
 
-Console.prototype.trace = function() {
+Console.prototype.trace = function trace() {
   // TODO probably can to do this better with V8's debug object once that is
   // exposed.
-  var err = new Error;
+  var err = new Error();
   err.name = 'Trace';
   err.message = util.format.apply(this, arguments);
-  Error.captureStackTrace(err, arguments.callee);
+  Error.captureStackTrace(err, trace);
   this.error(err.stack);
 };
 
