@@ -1,30 +1,19 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
 
-var NativeModule = require('native_module');
-var Script = process.binding('evals').NodeScript;
-var runInThisContext = Script.runInThisContext;
-var runInNewContext = Script.runInNewContext;
-var assert = require('assert').ok;
+const NativeModule = require('native_module');
+const util = require('util');
+const internalModule = require('internal/module');
+const internalUtil = require('internal/util');
+const runInThisContext = require('vm').runInThisContext;
+const assert = require('assert').ok;
+const fs = require('fs');
+const path = require('path');
+const internalModuleReadFile = process.binding('fs').internalModuleReadFile;
+const internalModuleStat = process.binding('fs').internalModuleStat;
 
+const splitRe = process.platform === 'win32' ? /[\/\\]/ : /\//;
+const isIndexRe = /^index\.\w+?$/;
+const shebangRe = /^\#\!.*/;
 
 // If obj.hasOwnProperty has been overridden, then calling
 // obj.hasOwnProperty(prop) will break.
@@ -48,9 +37,6 @@ function Module(id, parent) {
 }
 module.exports = Module;
 
-// Set the environ variable NODE_MODULE_CONTEXTS=1 to make node load all
-// modules in thier own context.
-Module._contextLoad = (+process.env['NODE_MODULE_CONTEXTS'] > 0);
 Module._cache = {};
 Module._pathCache = {};
 Module._extensions = {};
@@ -59,19 +45,10 @@ Module.globalPaths = [];
 
 Module.wrapper = NativeModule.wrapper;
 Module.wrap = NativeModule.wrap;
-
-var path = NativeModule.require('path');
-
-Module._debug = function() {};
-if (process.env.NODE_DEBUG && /module/.test(process.env.NODE_DEBUG)) {
-  Module._debug = function(x) {
-    console.error(x);
-  };
-}
-
+Module._debug = util.debuglog('module');
 
 // We use this alias for the preprocessor that filters it out
-var debug = Module._debug;
+const debug = Module._debug;
 
 
 // given a module name, and a list of paths to test, returns the first
@@ -85,27 +62,18 @@ var debug = Module._debug;
 //   -> a.<ext>
 //   -> a/index.<ext>
 
-function statPath(path) {
-  var fs = NativeModule.require('fs');
-  try {
-    return fs.statSync(path);
-  } catch (ex) {}
-  return false;
-}
-
 // check if the directory is a package.json dir
-var packageMainCache = {};
+const packageMainCache = {};
 
 function readPackage(requestPath) {
   if (hasOwnProperty(packageMainCache, requestPath)) {
     return packageMainCache[requestPath];
   }
 
-  var fs = NativeModule.require('fs');
-  try {
-    var jsonPath = path.resolve(requestPath, 'package.json');
-    var json = fs.readFileSync(jsonPath, 'utf8');
-  } catch (e) {
+  var jsonPath = path.resolve(requestPath, 'package.json');
+  var json = internalModuleReadFile(path._makeLong(jsonPath));
+
+  if (json === undefined) {
     return false;
   }
 
@@ -136,12 +104,12 @@ Module._realpathCache = {};
 
 // check if the file exists and is not a directory
 function tryFile(requestPath) {
-  var fs = NativeModule.require('fs');
-  var stats = statPath(requestPath);
-  if (stats && !stats.isDirectory()) {
-    return fs.realpathSync(requestPath, Module._realpathCache);
-  }
-  return false;
+  const rc = internalModuleStat(path._makeLong(requestPath));
+  return rc === 0 && toRealPath(requestPath);
+}
+
+function toRealPath(requestPath) {
+  return fs.realpathSync(requestPath, Module._realpathCache);
 }
 
 // given a path check a the file exists with any of the set extensions
@@ -156,11 +124,11 @@ function tryExtensions(p, exts) {
   return false;
 }
 
-
+var warned = false;
 Module._findPath = function(request, paths) {
   var exts = Object.keys(Module._extensions);
 
-  if (request.charAt(0) === '/') {
+  if (path.isAbsolute(request)) {
     paths = [''];
   }
 
@@ -173,14 +141,20 @@ Module._findPath = function(request, paths) {
 
   // For each path
   for (var i = 0, PL = paths.length; i < PL; i++) {
+    // Don't search further if path doesn't exist
+    if (paths[i] && internalModuleStat(path._makeLong(paths[i])) < 1) continue;
     var basePath = path.resolve(paths[i], request);
     var filename;
 
     if (!trailingSlash) {
-      // try to join the request to the path
-      filename = tryFile(basePath);
+      const rc = internalModuleStat(path._makeLong(basePath));
+      if (rc === 0) {  // File.
+        filename = toRealPath(basePath);
+      } else if (rc === 1) {  // Directory.
+        filename = tryPackage(basePath, exts);
+      }
 
-      if (!filename && !trailingSlash) {
+      if (!filename) {
         // try it with each of the extensions
         filename = tryExtensions(basePath, exts);
       }
@@ -196,6 +170,14 @@ Module._findPath = function(request, paths) {
     }
 
     if (filename) {
+      // Warn once if '.' resolved outside the module dir
+      if (request === '.' && i > 0) {
+        warned = internalUtil.printDeprecationMessage(
+          'warning: require(\'.\') resolved outside the package ' +
+          'directory. This functionality is deprecated and will be removed ' +
+          'soon.', warned);
+      }
+
       Module._pathCache[cacheKey] = filename;
       return filename;
     }
@@ -211,16 +193,13 @@ Module._nodeModulePaths = function(from) {
   // note: this approach *only* works when the path is guaranteed
   // to be absolute.  Doing a fully-edge-case-correct path.split
   // that works on both Windows and Posix is non-trivial.
-  var splitRe = process.platform === 'win32' ? /[\/\\]/ : /\//;
-  // yes, '/' works on both, but let's be a little canonical.
-  var joiner = process.platform === 'win32' ? '\\' : '/';
   var paths = [];
   var parts = from.split(splitRe);
 
   for (var tip = parts.length - 1; tip >= 0; tip--) {
     // don't search in .../node_modules/node_modules
     if (parts[tip] === 'node_modules') continue;
-    var dir = parts.slice(0, tip + 1).concat('node_modules').join(joiner);
+    var dir = parts.slice(0, tip + 1).concat('node_modules').join(path.sep);
     paths.push(dir);
   }
 
@@ -229,7 +208,7 @@ Module._nodeModulePaths = function(from) {
 
 
 Module._resolveLookupPaths = function(request, parent) {
-  if (NativeModule.exists(request)) {
+  if (NativeModule.nonInternalExists(request)) {
     return [request, []];
   }
 
@@ -240,6 +219,17 @@ Module._resolveLookupPaths = function(request, parent) {
       if (!parent.paths) parent.paths = [];
       paths = parent.paths.concat(paths);
     }
+
+    // Maintain backwards compat with certain broken uses of require('.')
+    // by putting the module's directory in front of the lookup paths.
+    if (request === '.') {
+      if (parent && parent.filename) {
+        paths.splice(0, 0, path.dirname(parent.filename));
+      } else {
+        paths.splice(0, 0, path.resolve(request));
+      }
+    }
+
     return [request, paths];
   }
 
@@ -255,7 +245,7 @@ Module._resolveLookupPaths = function(request, parent) {
   // Is the parent an index module?
   // We can assume the parent has a valid extension,
   // as it already has been accepted as a module.
-  var isIndex = /^index\.\w+?$/.test(path.basename(parent.filename));
+  var isIndex = isIndexRe.test(path.basename(parent.filename));
   var parentIdPath = isIndex ? parent.id : path.dirname(parent.id);
   var id = path.resolve(parentIdPath, request);
 
@@ -265,16 +255,23 @@ Module._resolveLookupPaths = function(request, parent) {
     id = './' + id;
   }
 
-  debug('RELATIVE: requested:' + request +
-        ' set ID to: ' + id + ' from ' + parent.id);
+  debug('RELATIVE: requested: %s set ID to: %s from %s', request, id,
+        parent.id);
 
   return [id, [path.dirname(parent.filename)]];
 };
 
 
+// Check the cache for the requested file.
+// 1. If a module already exists in the cache: return its exports object.
+// 2. If the module is native: call `NativeModule.require()` with the
+//    filename and return the result.
+// 3. Otherwise, create a new module for the file and save it to the cache.
+//    Then have it load  the file contents before returning its exports
+//    object.
 Module._load = function(request, parent, isMain) {
   if (parent) {
-    debug('Module._load REQUEST  ' + (request) + ' parent: ' + parent.id);
+    debug('Module._load REQUEST %s parent: %s', request, parent.id);
   }
 
   var filename = Module._resolveFilename(request, parent);
@@ -284,16 +281,8 @@ Module._load = function(request, parent, isMain) {
     return cachedModule.exports;
   }
 
-  if (NativeModule.exists(filename)) {
-    // REPL is a special case, because it needs the real require.
-    if (filename == 'repl') {
-      var replModule = new Module('repl');
-      replModule._compile(NativeModule.getSource('repl'), 'repl.js');
-      NativeModule._cache.repl = replModule;
-      return replModule.exports;
-    }
-
-    debug('load native module ' + request);
+  if (NativeModule.nonInternalExists(filename)) {
+    debug('load native module %s', request);
     return NativeModule.require(filename);
   }
 
@@ -321,7 +310,7 @@ Module._load = function(request, parent, isMain) {
 };
 
 Module._resolveFilename = function(request, parent) {
-  if (NativeModule.exists(request)) {
+  if (NativeModule.nonInternalExists(request)) {
     return request;
   }
 
@@ -330,8 +319,7 @@ Module._resolveFilename = function(request, parent) {
   var paths = resolvedModule[1];
 
   // look up the filename first, since that's the cache key.
-  debug('looking for ' + JSON.stringify(id) +
-        ' in ' + JSON.stringify(paths));
+  debug('looking for %j in %j', id, paths);
 
   var filename = Module._findPath(request, paths);
   if (!filename) {
@@ -343,9 +331,9 @@ Module._resolveFilename = function(request, parent) {
 };
 
 
+// Given a file name, pass it to the proper extension handler.
 Module.prototype.load = function(filename) {
-  debug('load ' + JSON.stringify(filename) +
-        ' for module ' + JSON.stringify(this.id));
+  debug('load %j for module %j', filename, this.id);
 
   assert(!this.loaded);
   this.filename = filename;
@@ -358,9 +346,11 @@ Module.prototype.load = function(filename) {
 };
 
 
+// Loads a module at the given file path. Returns that module's
+// `exports` property.
 Module.prototype.require = function(path) {
-  assert(typeof path === 'string', 'path must be a string');
   assert(path, 'missing path');
+  assert(typeof path === 'string', 'path must be a string');
   return Module._load(path, this);
 };
 
@@ -370,73 +360,19 @@ Module.prototype.require = function(path) {
 var resolvedArgv;
 
 
-// Returns exception if any
+// Run the file contents in the correct scope or sandbox. Expose
+// the correct helper variables (require, module, exports) to
+// the file.
+// Returns exception, if any.
 Module.prototype._compile = function(content, filename) {
-  var self = this;
   // remove shebang
-  content = content.replace(/^\#\!.*/, '');
-
-  function require(path) {
-    return self.require(path);
-  }
-
-  require.resolve = function(request) {
-    return Module._resolveFilename(request, self);
-  };
-
-  Object.defineProperty(require, 'paths', { get: function() {
-    throw new Error('require.paths is removed. Use ' +
-                    'node_modules folders, or the NODE_PATH ' +
-                    'environment variable instead.');
-  }});
-
-  require.main = process.mainModule;
-
-  // Enable support to add extra extension types
-  require.extensions = Module._extensions;
-  require.registerExtension = function() {
-    throw new Error('require.registerExtension() removed. Use ' +
-                    'require.extensions instead.');
-  };
-
-  require.cache = Module._cache;
-
-  var dirname = path.dirname(filename);
-
-  if (Module._contextLoad) {
-    if (self.id !== '.') {
-      debug('load submodule');
-      // not root module
-      var sandbox = {};
-      for (var k in global) {
-        sandbox[k] = global[k];
-      }
-      sandbox.require = require;
-      sandbox.exports = self.exports;
-      sandbox.__filename = filename;
-      sandbox.__dirname = dirname;
-      sandbox.module = self;
-      sandbox.global = sandbox;
-      sandbox.root = root;
-
-      return runInNewContext(content, sandbox, filename, true);
-    }
-
-    debug('load root module');
-    // root module
-    global.require = require;
-    global.exports = self.exports;
-    global.__filename = filename;
-    global.__dirname = dirname;
-    global.module = self;
-
-    return runInThisContext(content, filename, true);
-  }
+  content = content.replace(shebangRe, '');
 
   // create wrapper function
   var wrapper = Module.wrap(content);
 
-  var compiledWrapper = runInThisContext(wrapper, filename, true);
+  var compiledWrapper = runInThisContext(wrapper,
+                                      { filename: filename, lineOffset: 0 });
   if (global.v8debug) {
     if (!resolvedArgv) {
       // we enter the repl if we're not given a filename argument.
@@ -449,37 +385,44 @@ Module.prototype._compile = function(content, filename) {
 
     // Set breakpoint on module start
     if (filename === resolvedArgv) {
+      // Installing this dummy debug event listener tells V8 to start
+      // the debugger.  Without it, the setBreakPoint() fails with an
+      // 'illegal access' error.
+      global.v8debug.Debug.setListener(function() {});
       global.v8debug.Debug.setBreakPoint(compiledWrapper, 0, 0);
     }
   }
-  var args = [self.exports, require, self, filename, dirname];
-  return compiledWrapper.apply(self.exports, args);
+  const dirname = path.dirname(filename);
+  const require = internalModule.makeRequireFunction.call(this);
+
+  Object.defineProperty(require, 'paths', { get: function() {
+    throw new Error('require.paths is removed. Use ' +
+                    'node_modules folders, or the NODE_PATH ' +
+                    'environment variable instead.');
+  }});
+
+  require.registerExtension = function() {
+    throw new Error('require.registerExtension() removed. Use ' +
+                    'require.extensions instead.');
+  };
+
+  const args = [this.exports, require, this, filename, dirname];
+  return compiledWrapper.apply(this.exports, args);
 };
 
 
-function stripBOM(content) {
-  // Remove byte order marker. This catches EF BB BF (the UTF-8 BOM)
-  // because the buffer-to-string conversion in `fs.readFileSync()`
-  // translates it to FEFF, the UTF-16 BOM.
-  if (content.charCodeAt(0) === 0xFEFF) {
-    content = content.slice(1);
-  }
-  return content;
-}
-
-
 // Native extension for .js
 Module._extensions['.js'] = function(module, filename) {
-  var content = NativeModule.require('fs').readFileSync(filename, 'utf8');
-  module._compile(stripBOM(content), filename);
+  var content = fs.readFileSync(filename, 'utf8');
+  module._compile(internalModule.stripBOM(content), filename);
 };
 
 
 // Native extension for .json
 Module._extensions['.json'] = function(module, filename) {
-  var content = NativeModule.require('fs').readFileSync(filename, 'utf8');
+  var content = fs.readFileSync(filename, 'utf8');
   try {
-    module.exports = JSON.parse(stripBOM(content));
+    module.exports = JSON.parse(internalModule.stripBOM(content));
   } catch (err) {
     err.message = filename + ': ' + err.message;
     throw err;
@@ -488,7 +431,9 @@ Module._extensions['.json'] = function(module, filename) {
 
 
 //Native extension for .node
-Module._extensions['.node'] = process.dlopen;
+Module._extensions['.node'] = function(module, filename) {
+  return process.dlopen(module, path._makeLong(filename));
+};
 
 
 // bootstrap main module.
@@ -500,7 +445,7 @@ Module.runMain = function() {
 };
 
 Module._initPaths = function() {
-  var isWindows = process.platform === 'win32';
+  const isWindows = process.platform === 'win32';
 
   if (isWindows) {
     var homeDir = process.env.USERPROFILE;
@@ -515,9 +460,11 @@ Module._initPaths = function() {
     paths.unshift(path.resolve(homeDir, '.node_modules'));
   }
 
-  if (process.env['NODE_PATH']) {
-    var splitter = isWindows ? ';' : ':';
-    paths = process.env['NODE_PATH'].split(splitter).concat(paths);
+  var nodePath = process.env['NODE_PATH'];
+  if (nodePath) {
+    paths = nodePath.split(path.delimiter).filter(function(path) {
+      return !!path;
+    }).concat(paths);
   }
 
   modulePaths = paths;
@@ -526,9 +473,30 @@ Module._initPaths = function() {
   Module.globalPaths = modulePaths.slice(0);
 };
 
-// bootstrap repl
-Module.requireRepl = function() {
-  return Module._load('repl', '.');
+// TODO(bnoordhuis) Unused, remove in the future.
+Module.requireRepl = internalUtil.deprecate(function() {
+  return NativeModule.require('internal/repl');
+}, 'Module.requireRepl is deprecated.');
+
+Module._preloadModules = function(requests) {
+  if (!Array.isArray(requests))
+    return;
+
+  // Preloaded modules have a dummy parent module which is deemed to exist
+  // in the current working directory. This seeds the search path for
+  // preloaded modules.
+  var parent = new Module('internal/preload', null);
+  try {
+    parent.paths = Module._nodeModulePaths(process.cwd());
+  }
+  catch (e) {
+    if (e.code !== 'ENOENT') {
+      throw e;
+    }
+  }
+  requests.forEach(function(request) {
+    parent.require(request);
+  });
 };
 
 Module._initPaths();
