@@ -1,36 +1,18 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
-
 // Inspiration for this code comes from Salvatore Sanfilippo's linenoise.
 // https://github.com/antirez/linenoise
 // Reference:
 // * http://invisible-island.net/xterm/ctlseqs/ctlseqs.html
 // * http://www.3waylabs.com/nw/WWW/products/wizcon/vt220.html
 
-var kHistorySize = 30;
-var kBufSize = 10 * 1024;
+'use strict';
 
-var util = require('util');
-var inherits = require('util').inherits;
-var EventEmitter = require('events').EventEmitter;
+const kHistorySize = 30;
+
+const util = require('util');
+const internalUtil = require('internal/util');
+const inherits = util.inherits;
+const Buffer = require('buffer').Buffer;
+const EventEmitter = require('events');
 
 
 exports.createInterface = function(input, output, completer, terminal) {
@@ -46,30 +28,40 @@ exports.createInterface = function(input, output, completer, terminal) {
 
 function Interface(input, output, completer, terminal) {
   if (!(this instanceof Interface)) {
-    return new Interface(input, output, completer, terminal);
+    // call the constructor preserving original number of arguments
+    const self = Object.create(Interface.prototype);
+    Interface.apply(self, arguments);
+    return self;
   }
 
   this._sawReturn = false;
 
   EventEmitter.call(this);
+  var historySize;
 
   if (arguments.length === 1) {
     // an options object was given
     output = input.output;
     completer = input.completer;
     terminal = input.terminal;
+    historySize = input.historySize;
     input = input.input;
   }
+  historySize = historySize || kHistorySize;
 
-  completer = completer || function() { return []; };
-
-  if (typeof completer !== 'function') {
+  if (completer && typeof completer !== 'function') {
     throw new TypeError('Argument \'completer\' must be a function');
   }
 
+  if (typeof historySize !== 'number' ||
+      isNaN(historySize) ||
+      historySize < 0) {
+    throw new TypeError('Argument \'historySize\' must be a positive number');
+  }
+
   // backwards compat; check the isTTY prop of the output stream
   //  when `terminal` was not specified
-  if (typeof terminal == 'undefined') {
+  if (terminal === undefined && !(output === null || output === undefined)) {
     terminal = !!output.isTTY;
   }
 
@@ -77,11 +69,14 @@ function Interface(input, output, completer, terminal) {
 
   this.output = output;
   this.input = input;
+  this.historySize = historySize;
 
   // Check arity, 2 - for async, 1 for sync
-  this.completer = completer.length === 2 ? completer : function(v, callback) {
-    callback(null, completer(v));
-  };
+  if (typeof completer === 'function') {
+    this.completer = completer.length === 2 ? completer : function(v, cb) {
+      cb(null, completer(v));
+    };
+  }
 
   this.setPrompt('> ');
 
@@ -92,6 +87,17 @@ function Interface(input, output, completer, terminal) {
   }
 
   function onend() {
+    if (typeof self._line_buffer === 'string' &&
+        self._line_buffer.length > 0) {
+      self.emit('line', self._line_buffer);
+    }
+    self.close();
+  }
+
+  function ontermend() {
+    if (typeof self.line === 'string' && self.line.length > 0) {
+      self.emit('line', self.line);
+    }
     self.close();
   }
 
@@ -119,6 +125,7 @@ function Interface(input, output, completer, terminal) {
 
     // input usually refers to stdin
     input.on('keypress', onkeypress);
+    input.on('end', ontermend);
 
     // Current line
     this.line = '';
@@ -132,10 +139,15 @@ function Interface(input, output, completer, terminal) {
     this.history = [];
     this.historyIndex = -1;
 
-    output.on('resize', onresize);
+    if (output !== null && output !== undefined)
+      output.on('resize', onresize);
+
     self.once('close', function() {
       input.removeListener('keypress', onkeypress);
-      output.removeListener('resize', onresize);
+      input.removeListener('end', ontermend);
+      if (output !== null && output !== undefined) {
+        output.removeListener('resize', onresize);
+      }
     });
   }
 
@@ -145,18 +157,14 @@ function Interface(input, output, completer, terminal) {
 inherits(Interface, EventEmitter);
 
 Interface.prototype.__defineGetter__('columns', function() {
-  return this.output.columns || Infinity;
+  var columns = Infinity;
+  if (this.output && this.output.columns)
+    columns = this.output.columns;
+  return columns;
 });
 
-Interface.prototype.setPrompt = function(prompt, length) {
+Interface.prototype.setPrompt = function(prompt) {
   this._prompt = prompt;
-  if (length) {
-    this._promptLength = length;
-  } else {
-    var lines = prompt.split(/[\r\n]/);
-    var lastLine = lines[lines.length - 1];
-    this._promptLength = lastLine.length;
-  }
 };
 
 
@@ -173,7 +181,7 @@ Interface.prototype.prompt = function(preserveCursor) {
     if (!preserveCursor) this.cursor = 0;
     this._refreshLine();
   } else {
-    this.output.write(this._prompt);
+    this._writeToOutput(this._prompt);
   }
 };
 
@@ -203,6 +211,13 @@ Interface.prototype._onLine = function(line) {
   }
 };
 
+Interface.prototype._writeToOutput = function _writeToOutput(stringToWrite) {
+  if (typeof stringToWrite !== 'string')
+    throw new TypeError('stringToWrite must be a string');
+
+  if (this.output !== null && this.output !== undefined)
+    this.output.write(stringToWrite);
+};
 
 Interface.prototype._addHistory = function() {
   if (this.line.length === 0) return '';
@@ -211,7 +226,7 @@ Interface.prototype._addHistory = function() {
     this.history.unshift(this.line);
 
     // Only store so many
-    if (this.history.length > kHistorySize) this.history.pop();
+    if (this.history.length > this.historySize) this.history.pop();
   }
 
   this.historyIndex = -1;
@@ -220,13 +235,11 @@ Interface.prototype._addHistory = function() {
 
 
 Interface.prototype._refreshLine = function() {
-  var columns = this.columns;
-
   // line length
   var line = this._prompt + this.line;
-  var lineLength = line.length;
-  var lineCols = lineLength % columns;
-  var lineRows = (lineLength - lineCols) / columns;
+  var dispPos = this._getDisplayPos(line);
+  var lineCols = dispPos.cols;
+  var lineRows = dispPos.rows;
 
   // cursor position
   var cursorPos = this._getCursorPos();
@@ -243,11 +256,11 @@ Interface.prototype._refreshLine = function() {
   exports.clearScreenDown(this.output);
 
   // Write the prompt and the current buffer content.
-  this.output.write(line);
+  this._writeToOutput(line);
 
   // Force terminal to allocate a new line
   if (lineCols === 0) {
-    this.output.write(' ');
+    this._writeToOutput(' ');
   }
 
   // Move cursor to original position.
@@ -278,6 +291,7 @@ Interface.prototype.pause = function() {
   this.input.pause();
   this.paused = true;
   this.emit('pause');
+  return this;
 };
 
 
@@ -286,6 +300,7 @@ Interface.prototype.resume = function() {
   this.input.resume();
   this.paused = false;
   this.emit('resume');
+  return this;
 };
 
 
@@ -295,7 +310,7 @@ Interface.prototype.write = function(d, key) {
 };
 
 // \r\n, \n, or \r followed by something other than \n
-var lineEnding = /\r?\n|\r(?!\n)/;
+const lineEnding = /\r?\n|\r(?!\n)/;
 Interface.prototype._normalWrite = function(b) {
   if (b === undefined) {
     return;
@@ -306,11 +321,14 @@ Interface.prototype._normalWrite = function(b) {
     this._sawReturn = false;
   }
 
+  // Run test() on the new string chunk, not on the entire line buffer.
+  var newPartContainsEnding = lineEnding.test(string);
+
   if (this._line_buffer) {
     string = this._line_buffer + string;
     this._line_buffer = null;
   }
-  if (lineEnding.test(string)) {
+  if (newPartContainsEnding) {
     this._sawReturn = /\r$/.test(string);
 
     // got one or more newlines; process into "line" events
@@ -328,9 +346,6 @@ Interface.prototype._normalWrite = function(b) {
 };
 
 Interface.prototype._insertString = function(c) {
-  //BUG: Problem when adding tabs with following content.
-  //     Perhaps the bug is in _refreshLine(). Not sure.
-  //     A hack would be to insert spaces instead of literal '\t'.
   if (this.cursor < this.line.length) {
     var beg = this.line.slice(0, this.cursor);
     var end = this.line.slice(this.cursor, this.line.length);
@@ -344,7 +359,7 @@ Interface.prototype._insertString = function(c) {
     if (this._getCursorPos().cols === 0) {
       this._refreshLine();
     } else {
-      this.output.write(c);
+      this._writeToOutput(c);
     }
 
     // a hack to get the line refreshed if it's needed
@@ -371,11 +386,14 @@ Interface.prototype._tabComplete = function() {
       if (completions.length === 1) {
         self._insertString(completions[0].slice(completeOn.length));
       } else {
-        self.output.write('\r\n');
+        self._writeToOutput('\r\n');
         var width = completions.reduce(function(a, b) {
           return a.length > b.length ? a : b;
         }).length + 2;  // 2 space padding
-        var maxColumns = Math.floor(self.columns / width) || 1;
+        var maxColumns = Math.floor(self.columns / width);
+        if (!maxColumns || maxColumns === Infinity) {
+          maxColumns = 1;
+        }
         var group = [], c;
         for (var i = 0, compLen = completions.length; i < compLen; i++) {
           c = completions[i];
@@ -415,17 +433,17 @@ function handleGroup(self, group, width, maxColumns) {
         break;
       }
       var item = group[idx];
-      self.output.write(item);
+      self._writeToOutput(item);
       if (col < maxColumns - 1) {
         for (var s = 0, itemLen = item.length; s < width - itemLen;
              s++) {
-          self.output.write(' ');
+          self._writeToOutput(' ');
         }
       }
     }
-    self.output.write('\r\n');
+    self._writeToOutput('\r\n');
   }
-  self.output.write('\r\n');
+  self._writeToOutput('\r\n');
 }
 
 function commonPrefix(strings) {
@@ -518,7 +536,7 @@ Interface.prototype._deleteLineRight = function() {
 
 Interface.prototype.clearLine = function() {
   this._moveCursor(+Infinity);
-  this.output.write('\r\n');
+  this._writeToOutput('\r\n');
   this.line = '';
   this.cursor = 0;
   this.prevRows = 0;
@@ -559,12 +577,53 @@ Interface.prototype._historyPrev = function() {
 };
 
 
+// Returns the last character's display position of the given string
+Interface.prototype._getDisplayPos = function(str) {
+  var offset = 0;
+  var col = this.columns;
+  var row = 0;
+  var code;
+  str = stripVTControlCharacters(str);
+  for (var i = 0, len = str.length; i < len; i++) {
+    code = str.codePointAt(i);
+    if (code >= 0x10000) { // surrogates
+      i++;
+    }
+    if (code === 0x0a) { // new line \n
+      offset = 0;
+      row += 1;
+      continue;
+    }
+    if (isFullWidthCodePoint(code)) {
+      if ((offset + 1) % col === 0) {
+        offset++;
+      }
+      offset += 2;
+    } else {
+      offset++;
+    }
+  }
+  var cols = offset % col;
+  var rows = row + (offset - cols) / col;
+  return {cols: cols, rows: rows};
+};
+
+
 // Returns current cursor's position and line
 Interface.prototype._getCursorPos = function() {
   var columns = this.columns;
-  var cursorPos = this.cursor + this._promptLength;
-  var cols = cursorPos % columns;
-  var rows = (cursorPos - cols) / columns;
+  var strBeforeCursor = this._prompt + this.line.substring(0, this.cursor);
+  var dispPos = this._getDisplayPos(stripVTControlCharacters(strBeforeCursor));
+  var cols = dispPos.cols;
+  var rows = dispPos.rows;
+  // If the cursor is on a full-width character which steps over the line,
+  // move the cursor to the beginning of the next line.
+  if (cols + 1 === columns &&
+      this.cursor < this.line.length &&
+      isFullWidthCodePoint(this.line.codePointAt(this.cursor))) {
+    rows++;
+    cols = 0;
+  }
   return {cols: cols, rows: rows};
 };
 
@@ -578,13 +637,24 @@ Interface.prototype._moveCursor = function(dx) {
 
   // bounds check
   if (this.cursor < 0) this.cursor = 0;
-  if (this.cursor > this.line.length) this.cursor = this.line.length;
+  else if (this.cursor > this.line.length) this.cursor = this.line.length;
 
   var newPos = this._getCursorPos();
 
   // check if cursors are in the same line
   if (oldPos.rows === newPos.rows) {
-    exports.moveCursor(this.output, this.cursor - oldcursor, 0);
+    var diffCursor = this.cursor - oldcursor;
+    var diffWidth;
+    if (diffCursor < 0) {
+      diffWidth = -getStringWidth(
+          this.line.substring(this.cursor, oldcursor)
+          );
+    } else if (diffCursor > 0) {
+      diffWidth = getStringWidth(
+          this.line.substring(this.cursor, oldcursor)
+          );
+    }
+    exports.moveCursor(this.output, diffWidth, 0);
     this.prevRows = newPos.rows;
   } else {
     this._refreshLine();
@@ -616,7 +686,7 @@ Interface.prototype._ttyWrite = function(s, key) {
 
     switch (key.name) {
       case 'c':
-        if (EventEmitter.listenerCount(this, 'SIGINT') > 0) {
+        if (this.listenerCount('SIGINT') > 0) {
           this.emit('SIGINT');
         } else {
           // This readline instance is finished
@@ -679,7 +749,7 @@ Interface.prototype._ttyWrite = function(s, key) {
 
       case 'z':
         if (process.platform == 'win32') break;
-        if (EventEmitter.listenerCount(this, 'SIGTSTP') > 0) {
+        if (this.listenerCount('SIGTSTP') > 0) {
           this.emit('SIGTSTP');
         } else {
           process.once('SIGCONT', (function(self) {
@@ -691,7 +761,7 @@ Interface.prototype._ttyWrite = function(s, key) {
                 self.pause();
                 self.emit('SIGCONT');
               }
-              // explictly re-enable "raw mode" and move the cursor to
+              // explicitly re-enable "raw mode" and move the cursor to
               // the correct position.
               // See https://github.com/joyent/node/issues/3295.
               self._setRawMode(true);
@@ -712,10 +782,6 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._deleteWordRight();
         break;
 
-      case 'backspace':
-        this._deleteWordLeft();
-        break;
-
       case 'left':
         this._wordLeft();
         break;
@@ -775,10 +841,6 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._deleteRight();
         break;
 
-      case 'tab': // tab completion
-        this._tabComplete();
-        break;
-
       case 'left':
         this._moveCursor(-1);
         break;
@@ -803,8 +865,16 @@ Interface.prototype._ttyWrite = function(s, key) {
         this._historyNext();
         break;
 
+      case 'tab':
+        // If tab completion enabled, do that...
+        if (typeof this.completer === 'function') {
+          this._tabComplete();
+          break;
+        }
+        // falls through
+
       default:
-        if (Buffer.isBuffer(s))
+        if (s instanceof Buffer)
           s = s.toString('utf-8');
 
         if (s) {
@@ -824,20 +894,37 @@ Interface.prototype._ttyWrite = function(s, key) {
 exports.Interface = Interface;
 
 
-
 /**
  * accepts a readable Stream instance and makes it emit "keypress" events
  */
 
+const KEYPRESS_DECODER = Symbol('keypress-decoder');
+const ESCAPE_DECODER = Symbol('escape-decoder');
+
 function emitKeypressEvents(stream) {
-  if (stream._keypressDecoder) return;
+  if (stream[KEYPRESS_DECODER]) return;
   var StringDecoder = require('string_decoder').StringDecoder; // lazy load
-  stream._keypressDecoder = new StringDecoder('utf8');
+  stream[KEYPRESS_DECODER] = new StringDecoder('utf8');
+
+  stream[ESCAPE_DECODER] = emitKeys(stream);
+  stream[ESCAPE_DECODER].next();
 
   function onData(b) {
-    if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
-      var r = stream._keypressDecoder.write(b);
-      if (r) emitKey(stream, r);
+    if (stream.listenerCount('keypress') > 0) {
+      var r = stream[KEYPRESS_DECODER].write(b);
+      if (r) {
+        for (var i = 0; i < r.length; i++) {
+          try {
+            stream[ESCAPE_DECODER].next(r[i]);
+          } catch (err) {
+            // if the generator throws (it could happen in the `keypress`
+            // event), we need to restart it.
+            stream[ESCAPE_DECODER] = emitKeys(stream);
+            stream[ESCAPE_DECODER].next();
+            throw err;
+          }
+        }
+      }
     } else {
       // Nobody's watching anyway
       stream.removeListener('data', onData);
@@ -852,7 +939,7 @@ function emitKeypressEvents(stream) {
     }
   }
 
-  if (EventEmitter.listenerCount(stream, 'keypress') > 0) {
+  if (stream.listenerCount('keypress') > 0) {
     stream.on('data', onData);
   } else {
     stream.on('newListener', onNewListener);
@@ -889,211 +976,285 @@ exports.emitKeypressEvents = emitKeypressEvents;
 */
 
 // Regexes used for ansi escape code splitting
-var metaKeyCodeRe = /^(?:\x1b)([a-zA-Z0-9])$/;
-var functionKeyCodeRe =
-    /^(?:\x1b+)(O|N|\[|\[\[)(?:(\d+)(?:;(\d+))?([~^$])|(?:1;)?(\d+)?([a-zA-Z]))/;
+const metaKeyCodeReAnywhere = /(?:\x1b)([a-zA-Z0-9])/;
+const functionKeyCodeReAnywhere = new RegExp('(?:\x1b+)(O|N|\\[|\\[\\[)(?:' + [
+  '(\\d+)(?:;(\\d+))?([~^$])',
+  '(?:M([@ #!a`])(.)(.))', // mouse
+  '(?:1;)?(\\d+)?([a-zA-Z])'
+].join('|') + ')');
 
-function emitKey(stream, s) {
-  var ch,
-      key = {
-        name: undefined,
-        ctrl: false,
-        meta: false,
-        shift: false
-      },
-      parts;
 
-  if (Buffer.isBuffer(s)) {
-    if (s[0] > 127 && s[1] === undefined) {
-      s[0] -= 128;
-      s = '\x1b' + s.toString(stream.encoding || 'utf-8');
+function* emitKeys(stream) {
+  while (true) {
+    var ch = yield;
+    var s = ch;
+    var escaped = false;
+    var key = {
+      sequence: null,
+      name: undefined,
+      ctrl: false,
+      meta: false,
+      shift: false
+    };
+
+    if (ch === '\x1b') {
+      escaped = true;
+      s += (ch = yield);
+
+      if (ch === '\x1b') {
+        s += (ch = yield);
+      }
+    }
+
+    if (escaped && (ch === 'O' || ch === '[')) {
+      // ansi escape sequence
+      var code = ch;
+      var modifier = 0;
+
+      if (ch === 'O') {
+        // ESC O letter
+        // ESC O modifier letter
+        s += (ch = yield);
+
+        if (ch >= '0' && ch <= '9') {
+          modifier = (ch >> 0) - 1;
+          s += (ch = yield);
+        }
+
+        code += ch;
+
+      } else if (ch === '[') {
+        // ESC [ letter
+        // ESC [ modifier letter
+        // ESC [ [ modifier letter
+        // ESC [ [ num char
+        s += (ch = yield);
+
+        if (ch === '[') {
+          // \x1b[[A
+          //      ^--- escape codes might have a second bracket
+          code += ch;
+          s += (ch = yield);
+        }
+
+        /*
+         * Here and later we try to buffer just enough data to get
+         * a complete ascii sequence.
+         *
+         * We have basically two classes of ascii characters to process:
+         *
+         *
+         * 1. `\x1b[24;5~` should be parsed as { code: '[24~', modifier: 5 }
+         *
+         * This particular example is featuring Ctrl+F12 in xterm.
+         *
+         *  - `;5` part is optional, e.g. it could be `\x1b[24~`
+         *  - first part can contain one or two digits
+         *
+         * So the generic regexp is like /^\d\d?(;\d)?[~^$]$/
+         *
+         *
+         * 2. `\x1b[1;5H` should be parsed as { code: '[H', modifier: 5 }
+         *
+         * This particular example is featuring Ctrl+Home in xterm.
+         *
+         *  - `1;5` part is optional, e.g. it could be `\x1b[H`
+         *  - `1;` part is optional, e.g. it could be `\x1b[5H`
+         *
+         * So the generic regexp is like /^((\d;)?\d)?[A-Za-z]$/
+         *
+         */
+        const cmdStart = s.length - 1;
+
+        // skip one or two leading digits
+        if (ch >= '0' && ch <= '9') {
+          s += (ch = yield);
+
+          if (ch >= '0' && ch <= '9') {
+            s += (ch = yield);
+          }
+        }
+
+        // skip modifier
+        if (ch === ';') {
+          s += (ch = yield);
+
+          if (ch >= '0' && ch <= '9') {
+            s += (ch = yield);
+          }
+        }
+
+        /*
+         * We buffered enough data, now trying to extract code
+         * and modifier from it
+         */
+        const cmd = s.slice(cmdStart);
+        var match;
+
+        if ((match = cmd.match(/^(\d\d?)(;(\d))?([~^$])$/))) {
+          code += match[1] + match[4];
+          modifier = (match[3] || 1) - 1;
+        } else if ((match = cmd.match(/^((\d;)?(\d))?([A-Za-z])$/))) {
+          code += match[4];
+          modifier = (match[3] || 1) - 1;
+        } else {
+          code += cmd;
+        }
+      }
+
+      // Parse the key modifier
+      key.ctrl = !!(modifier & 4);
+      key.meta = !!(modifier & 10);
+      key.shift = !!(modifier & 1);
+      key.code = code;
+
+      // Parse the key itself
+      switch (code) {
+        /* xterm/gnome ESC O letter */
+        case 'OP': key.name = 'f1'; break;
+        case 'OQ': key.name = 'f2'; break;
+        case 'OR': key.name = 'f3'; break;
+        case 'OS': key.name = 'f4'; break;
+
+        /* xterm/rxvt ESC [ number ~ */
+        case '[11~': key.name = 'f1'; break;
+        case '[12~': key.name = 'f2'; break;
+        case '[13~': key.name = 'f3'; break;
+        case '[14~': key.name = 'f4'; break;
+
+        /* from Cygwin and used in libuv */
+        case '[[A': key.name = 'f1'; break;
+        case '[[B': key.name = 'f2'; break;
+        case '[[C': key.name = 'f3'; break;
+        case '[[D': key.name = 'f4'; break;
+        case '[[E': key.name = 'f5'; break;
+
+        /* common */
+        case '[15~': key.name = 'f5'; break;
+        case '[17~': key.name = 'f6'; break;
+        case '[18~': key.name = 'f7'; break;
+        case '[19~': key.name = 'f8'; break;
+        case '[20~': key.name = 'f9'; break;
+        case '[21~': key.name = 'f10'; break;
+        case '[23~': key.name = 'f11'; break;
+        case '[24~': key.name = 'f12'; break;
+
+        /* xterm ESC [ letter */
+        case '[A': key.name = 'up'; break;
+        case '[B': key.name = 'down'; break;
+        case '[C': key.name = 'right'; break;
+        case '[D': key.name = 'left'; break;
+        case '[E': key.name = 'clear'; break;
+        case '[F': key.name = 'end'; break;
+        case '[H': key.name = 'home'; break;
+
+        /* xterm/gnome ESC O letter */
+        case 'OA': key.name = 'up'; break;
+        case 'OB': key.name = 'down'; break;
+        case 'OC': key.name = 'right'; break;
+        case 'OD': key.name = 'left'; break;
+        case 'OE': key.name = 'clear'; break;
+        case 'OF': key.name = 'end'; break;
+        case 'OH': key.name = 'home'; break;
+
+        /* xterm/rxvt ESC [ number ~ */
+        case '[1~': key.name = 'home'; break;
+        case '[2~': key.name = 'insert'; break;
+        case '[3~': key.name = 'delete'; break;
+        case '[4~': key.name = 'end'; break;
+        case '[5~': key.name = 'pageup'; break;
+        case '[6~': key.name = 'pagedown'; break;
+
+        /* putty */
+        case '[[5~': key.name = 'pageup'; break;
+        case '[[6~': key.name = 'pagedown'; break;
+
+        /* rxvt */
+        case '[7~': key.name = 'home'; break;
+        case '[8~': key.name = 'end'; break;
+
+        /* rxvt keys with modifiers */
+        case '[a': key.name = 'up'; key.shift = true; break;
+        case '[b': key.name = 'down'; key.shift = true; break;
+        case '[c': key.name = 'right'; key.shift = true; break;
+        case '[d': key.name = 'left'; key.shift = true; break;
+        case '[e': key.name = 'clear'; key.shift = true; break;
+
+        case '[2$': key.name = 'insert'; key.shift = true; break;
+        case '[3$': key.name = 'delete'; key.shift = true; break;
+        case '[5$': key.name = 'pageup'; key.shift = true; break;
+        case '[6$': key.name = 'pagedown'; key.shift = true; break;
+        case '[7$': key.name = 'home'; key.shift = true; break;
+        case '[8$': key.name = 'end'; key.shift = true; break;
+
+        case 'Oa': key.name = 'up'; key.ctrl = true; break;
+        case 'Ob': key.name = 'down'; key.ctrl = true; break;
+        case 'Oc': key.name = 'right'; key.ctrl = true; break;
+        case 'Od': key.name = 'left'; key.ctrl = true; break;
+        case 'Oe': key.name = 'clear'; key.ctrl = true; break;
+
+        case '[2^': key.name = 'insert'; key.ctrl = true; break;
+        case '[3^': key.name = 'delete'; key.ctrl = true; break;
+        case '[5^': key.name = 'pageup'; key.ctrl = true; break;
+        case '[6^': key.name = 'pagedown'; key.ctrl = true; break;
+        case '[7^': key.name = 'home'; key.ctrl = true; break;
+        case '[8^': key.name = 'end'; key.ctrl = true; break;
+
+        /* misc. */
+        case '[Z': key.name = 'tab'; key.shift = true; break;
+        default: key.name = 'undefined'; break;
+      }
+
+    } else if (ch === '\r') {
+      // carriage return
+      key.name = 'return';
+
+    } else if (ch === '\n') {
+      // enter, should have been called linefeed
+      key.name = 'enter';
+
+    } else if (ch === '\t') {
+      // tab
+      key.name = 'tab';
+
+    } else if (ch === '\b' || ch === '\x7f') {
+      // backspace or ctrl+h
+      key.name = 'backspace';
+      key.meta = escaped;
+
+    } else if (ch === '\x1b') {
+      // escape key
+      key.name = 'escape';
+      key.meta = escaped;
+
+    } else if (ch === ' ') {
+      key.name = 'space';
+      key.meta = escaped;
+
+    } else if (!escaped && ch <= '\x1a') {
+      // ctrl+letter
+      key.name = String.fromCharCode(ch.charCodeAt(0) + 'a'.charCodeAt(0) - 1);
+      key.ctrl = true;
+
+    } else if (/^[0-9A-Za-z]$/.test(ch)) {
+      // letter, number, shift+letter
+      key.name = ch.toLowerCase();
+      key.shift = /^[A-Z]$/.test(ch);
+      key.meta = escaped;
+    }
+
+    key.sequence = s;
+
+    if (key.name !== undefined) {
+      /* Named character or sequence */
+      stream.emit('keypress', escaped ? undefined : s, key);
+    } else if (s.length === 1) {
+      /* Single unnamed character, e.g. "." */
+      stream.emit('keypress', s);
     } else {
-      s = s.toString(stream.encoding || 'utf-8');
+      /* Unrecognized or broken escape sequence, don't emit anything */
     }
   }
-
-  key.sequence = s;
-
-  if (s === '\r') {
-    // carriage return
-    key.name = 'return';
-
-  } else if (s === '\n') {
-    // enter, should have been called linefeed
-    key.name = 'enter';
-
-  } else if (s === '\t') {
-    // tab
-    key.name = 'tab';
-
-  } else if (s === '\b' || s === '\x7f' ||
-             s === '\x1b\x7f' || s === '\x1b\b') {
-    // backspace or ctrl+h
-    key.name = 'backspace';
-    key.meta = (s.charAt(0) === '\x1b');
-
-  } else if (s === '\x1b' || s === '\x1b\x1b') {
-    // escape key
-    key.name = 'escape';
-    key.meta = (s.length === 2);
-
-  } else if (s === ' ' || s === '\x1b ') {
-    key.name = 'space';
-    key.meta = (s.length === 2);
-
-  } else if (s.length === 1 && s <= '\x1a') {
-    // ctrl+letter
-    key.name = String.fromCharCode(s.charCodeAt(0) + 'a'.charCodeAt(0) - 1);
-    key.ctrl = true;
-
-  } else if (s.length === 1 && s >= 'a' && s <= 'z') {
-    // lowercase letter
-    key.name = s;
-
-  } else if (s.length === 1 && s >= 'A' && s <= 'Z') {
-    // shift+letter
-    key.name = s.toLowerCase();
-    key.shift = true;
-
-  } else if (parts = metaKeyCodeRe.exec(s)) {
-    // meta+character key
-    key.name = parts[1].toLowerCase();
-    key.meta = true;
-    key.shift = /^[A-Z]$/.test(parts[1]);
-
-  } else if (parts = functionKeyCodeRe.exec(s)) {
-    // ansi escape sequence
-
-    // reassemble the key code leaving out leading \x1b's,
-    // the modifier key bitflag and any meaningless "1;" sequence
-    var code = (parts[1] || '') + (parts[2] || '') +
-               (parts[4] || '') + (parts[6] || ''),
-        modifier = (parts[3] || parts[5] || 1) - 1;
-
-    // Parse the key modifier
-    key.ctrl = !!(modifier & 4);
-    key.meta = !!(modifier & 10);
-    key.shift = !!(modifier & 1);
-    key.code = code;
-
-    // Parse the key itself
-    switch (code) {
-      /* xterm/gnome ESC O letter */
-      case 'OP': key.name = 'f1'; break;
-      case 'OQ': key.name = 'f2'; break;
-      case 'OR': key.name = 'f3'; break;
-      case 'OS': key.name = 'f4'; break;
-
-      /* xterm/rxvt ESC [ number ~ */
-      case '[11~': key.name = 'f1'; break;
-      case '[12~': key.name = 'f2'; break;
-      case '[13~': key.name = 'f3'; break;
-      case '[14~': key.name = 'f4'; break;
-
-      /* from Cygwin and used in libuv */
-      case '[[A': key.name = 'f1'; break;
-      case '[[B': key.name = 'f2'; break;
-      case '[[C': key.name = 'f3'; break;
-      case '[[D': key.name = 'f4'; break;
-      case '[[E': key.name = 'f5'; break;
-
-      /* common */
-      case '[15~': key.name = 'f5'; break;
-      case '[17~': key.name = 'f6'; break;
-      case '[18~': key.name = 'f7'; break;
-      case '[19~': key.name = 'f8'; break;
-      case '[20~': key.name = 'f9'; break;
-      case '[21~': key.name = 'f10'; break;
-      case '[23~': key.name = 'f11'; break;
-      case '[24~': key.name = 'f12'; break;
-
-      /* xterm ESC [ letter */
-      case '[A': key.name = 'up'; break;
-      case '[B': key.name = 'down'; break;
-      case '[C': key.name = 'right'; break;
-      case '[D': key.name = 'left'; break;
-      case '[E': key.name = 'clear'; break;
-      case '[F': key.name = 'end'; break;
-      case '[H': key.name = 'home'; break;
-
-      /* xterm/gnome ESC O letter */
-      case 'OA': key.name = 'up'; break;
-      case 'OB': key.name = 'down'; break;
-      case 'OC': key.name = 'right'; break;
-      case 'OD': key.name = 'left'; break;
-      case 'OE': key.name = 'clear'; break;
-      case 'OF': key.name = 'end'; break;
-      case 'OH': key.name = 'home'; break;
-
-      /* xterm/rxvt ESC [ number ~ */
-      case '[1~': key.name = 'home'; break;
-      case '[2~': key.name = 'insert'; break;
-      case '[3~': key.name = 'delete'; break;
-      case '[4~': key.name = 'end'; break;
-      case '[5~': key.name = 'pageup'; break;
-      case '[6~': key.name = 'pagedown'; break;
-
-      /* putty */
-      case '[[5~': key.name = 'pageup'; break;
-      case '[[6~': key.name = 'pagedown'; break;
-
-      /* rxvt */
-      case '[7~': key.name = 'home'; break;
-      case '[8~': key.name = 'end'; break;
-
-      /* rxvt keys with modifiers */
-      case '[a': key.name = 'up'; key.shift = true; break;
-      case '[b': key.name = 'down'; key.shift = true; break;
-      case '[c': key.name = 'right'; key.shift = true; break;
-      case '[d': key.name = 'left'; key.shift = true; break;
-      case '[e': key.name = 'clear'; key.shift = true; break;
-
-      case '[2$': key.name = 'insert'; key.shift = true; break;
-      case '[3$': key.name = 'delete'; key.shift = true; break;
-      case '[5$': key.name = 'pageup'; key.shift = true; break;
-      case '[6$': key.name = 'pagedown'; key.shift = true; break;
-      case '[7$': key.name = 'home'; key.shift = true; break;
-      case '[8$': key.name = 'end'; key.shift = true; break;
-
-      case 'Oa': key.name = 'up'; key.ctrl = true; break;
-      case 'Ob': key.name = 'down'; key.ctrl = true; break;
-      case 'Oc': key.name = 'right'; key.ctrl = true; break;
-      case 'Od': key.name = 'left'; key.ctrl = true; break;
-      case 'Oe': key.name = 'clear'; key.ctrl = true; break;
-
-      case '[2^': key.name = 'insert'; key.ctrl = true; break;
-      case '[3^': key.name = 'delete'; key.ctrl = true; break;
-      case '[5^': key.name = 'pageup'; key.ctrl = true; break;
-      case '[6^': key.name = 'pagedown'; key.ctrl = true; break;
-      case '[7^': key.name = 'home'; key.ctrl = true; break;
-      case '[8^': key.name = 'end'; key.ctrl = true; break;
-
-      /* misc. */
-      case '[Z': key.name = 'tab'; key.shift = true; break;
-      default: key.name = 'undefined'; break;
-
-    }
-  } else if (s.length > 1 && s[0] !== '\x1b') {
-    // Got a longer-than-one string of characters.
-    // Probably a paste, since it wasn't a control sequence.
-    Array.prototype.forEach.call(s, function(c) {
-      emitKey(stream, c);
-    });
-    return;
-  }
-
-  // Don't emit a key if no name was found
-  if (key.name === undefined) {
-    key = undefined;
-  }
-
-  if (s.length === 1) {
-    ch = s;
-  }
-
-  if (key || ch) {
-    stream.emit('keypress', ch, key);
-  }
 }
 
 
@@ -1102,6 +1263,9 @@ function emitKey(stream, s) {
  */
 
 function cursorTo(stream, x, y) {
+  if (stream === null || stream === undefined)
+    return;
+
   if (typeof x !== 'number' && typeof y !== 'number')
     return;
 
@@ -1122,6 +1286,9 @@ exports.cursorTo = cursorTo;
  */
 
 function moveCursor(stream, dx, dy) {
+  if (stream === null || stream === undefined)
+    return;
+
   if (dx < 0) {
     stream.write('\x1b[' + (-dx) + 'D');
   } else if (dx > 0) {
@@ -1145,6 +1312,9 @@ exports.moveCursor = moveCursor;
  */
 
 function clearLine(stream, dir) {
+  if (stream === null || stream === undefined)
+    return;
+
   if (dir < 0) {
     // to the beginning
     stream.write('\x1b[1K');
@@ -1164,6 +1334,113 @@ exports.clearLine = clearLine;
  */
 
 function clearScreenDown(stream) {
+  if (stream === null || stream === undefined)
+    return;
+
   stream.write('\x1b[0J');
 }
 exports.clearScreenDown = clearScreenDown;
+
+
+/**
+ * Returns the number of columns required to display the given string.
+ */
+
+function getStringWidth(str) {
+  var width = 0;
+  str = stripVTControlCharacters(str);
+  for (var i = 0, len = str.length; i < len; i++) {
+    var code = str.codePointAt(i);
+    if (code >= 0x10000) { // surrogates
+      i++;
+    }
+    if (isFullWidthCodePoint(code)) {
+      width += 2;
+    } else {
+      width++;
+    }
+  }
+  return width;
+}
+exports.getStringWidth = getStringWidth;
+
+
+/**
+ * Returns true if the character represented by a given
+ * Unicode code point is full-width. Otherwise returns false.
+ */
+
+function isFullWidthCodePoint(code) {
+  if (isNaN(code)) {
+    return false;
+  }
+
+  // Code points are derived from:
+  // http://www.unicode.org/Public/UNIDATA/EastAsianWidth.txt
+  if (code >= 0x1100 && (
+      code <= 0x115f ||  // Hangul Jamo
+      0x2329 === code || // LEFT-POINTING ANGLE BRACKET
+      0x232a === code || // RIGHT-POINTING ANGLE BRACKET
+      // CJK Radicals Supplement .. Enclosed CJK Letters and Months
+      (0x2e80 <= code && code <= 0x3247 && code !== 0x303f) ||
+      // Enclosed CJK Letters and Months .. CJK Unified Ideographs Extension A
+      0x3250 <= code && code <= 0x4dbf ||
+      // CJK Unified Ideographs .. Yi Radicals
+      0x4e00 <= code && code <= 0xa4c6 ||
+      // Hangul Jamo Extended-A
+      0xa960 <= code && code <= 0xa97c ||
+      // Hangul Syllables
+      0xac00 <= code && code <= 0xd7a3 ||
+      // CJK Compatibility Ideographs
+      0xf900 <= code && code <= 0xfaff ||
+      // Vertical Forms
+      0xfe10 <= code && code <= 0xfe19 ||
+      // CJK Compatibility Forms .. Small Form Variants
+      0xfe30 <= code && code <= 0xfe6b ||
+      // Halfwidth and Fullwidth Forms
+      0xff01 <= code && code <= 0xff60 ||
+      0xffe0 <= code && code <= 0xffe6 ||
+      // Kana Supplement
+      0x1b000 <= code && code <= 0x1b001 ||
+      // Enclosed Ideographic Supplement
+      0x1f200 <= code && code <= 0x1f251 ||
+      // CJK Unified Ideographs Extension B .. Tertiary Ideographic Plane
+      0x20000 <= code && code <= 0x3fffd)) {
+    return true;
+  }
+  return false;
+}
+exports.isFullWidthCodePoint = isFullWidthCodePoint;
+
+
+/**
+ * Returns the Unicode code point for the character at the
+ * given index in the given string. Similar to String.charCodeAt(),
+ * but this function handles surrogates (code point >= 0x10000).
+ */
+
+function codePointAt(str, index) {
+  var code = str.charCodeAt(index);
+  var low;
+  if (0xd800 <= code && code <= 0xdbff) { // High surrogate
+    low = str.charCodeAt(index + 1);
+    if (!isNaN(low)) {
+      code = 0x10000 + (code - 0xd800) * 0x400 + (low - 0xdc00);
+    }
+  }
+  return code;
+}
+exports.codePointAt = internalUtil.deprecate(codePointAt,
+    'readline.codePointAt is deprecated. ' +
+    'Use String.prototype.codePointAt instead.');
+
+
+/**
+ * Tries to remove all VT control characters. Use to estimate displayed
+ * string width. May be buggy due to not running a real state machine
+ */
+function stripVTControlCharacters(str) {
+  str = str.replace(new RegExp(functionKeyCodeReAnywhere.source, 'g'), '');
+  return str.replace(new RegExp(metaKeyCodeReAnywhere.source, 'g'), '');
+}
+exports.stripVTControlCharacters = stripVTControlCharacters;
