@@ -1,30 +1,13 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
+'use strict';
+
+const util = require('util');
+const internalUtil = require('internal/util');
+const net = require('net');
+const TTY = process.binding('tty_wrap').TTY;
+const isTTY = process.binding('tty_wrap').isTTY;
+const inherits = util.inherits;
+const errnoException = util._errnoException;
 
-var assert = require('assert');
-var inherits = require('util').inherits;
-var net = require('net');
-var TTY = process.binding('tty_wrap').TTY;
-var isTTY = process.binding('tty_wrap').isTTY;
-var util = require('util');
 
 exports.isatty = function(fd) {
   return isTTY(fd);
@@ -32,12 +15,13 @@ exports.isatty = function(fd) {
 
 
 // backwards-compat
-exports.setRawMode = util.deprecate(function(flag) {
+exports.setRawMode = internalUtil.deprecate(function(flag) {
   if (!process.stdin.isTTY) {
     throw new Error('can\'t set raw mode on non-tty');
   }
   process.stdin.setRawMode(flag);
-}, 'tty.setRawMode: Use `process.stdin.setRawMode()` instead.');
+}, 'tty.setRawMode is deprecated. ' +
+   'Use process.stdin.setRawMode instead.');
 
 
 function ReadStream(fd, options) {
@@ -67,7 +51,6 @@ ReadStream.prototype.setRawMode = function(flag) {
 };
 
 
-
 function WriteStream(fd) {
   if (!(this instanceof WriteStream)) return new WriteStream(fd);
   net.Socket.call(this, {
@@ -76,8 +59,9 @@ function WriteStream(fd) {
     writable: true
   });
 
-  var winSize = this._handle.getWindowSize();
-  if (winSize) {
+  var winSize = [];
+  var err = this._handle.getWindowSize(winSize);
+  if (!err) {
     this.columns = winSize[0];
     this.rows = winSize[1];
   }
@@ -92,9 +76,10 @@ WriteStream.prototype.isTTY = true;
 WriteStream.prototype._refreshSize = function() {
   var oldCols = this.columns;
   var oldRows = this.rows;
-  var winSize = this._handle.getWindowSize();
-  if (!winSize) {
-    this.emit('error', errnoException(process._errno, 'getWindowSize'));
+  var winSize = [];
+  var err = this._handle.getWindowSize(winSize);
+  if (err) {
+    this.emit('error', errnoException(err, 'getWindowSize'));
     return;
   }
   var newCols = winSize[0];
@@ -123,12 +108,3 @@ WriteStream.prototype.clearScreenDown = function() {
 WriteStream.prototype.getWindowSize = function() {
   return [this.columns, this.rows];
 };
-
-
-// TODO share with net_uv and others
-function errnoException(errorno, syscall) {
-  var e = new Error(syscall + ' ' + errorno);
-  e.errno = e.code = errorno;
-  e.syscall = syscall;
-  return e;
-}
