@@ -1,10 +1,32 @@
+/** Copyright Joyent, Inc. and other Node contributors.
+*
+* Permission is hereby granted, free of charge, to any person obtaining a
+* copy of this software and associated documentation files (the
+* "Software"), to deal in the Software without restriction, including
+* without limitation the rights to use, copy, modify, merge, publish,
+* distribute, sublicense, and/or sell copies of the Software, and to permit
+* persons to whom the Software is furnished to do so, subject to the
+* following conditions:
+*
+* The above copyright notice and this permission notice shall be included
+* in all copies or substantial portions of the Software.
+*
+* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+* NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+* OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+* USE OR OTHER DEALINGS IN THE SOFTWARE.
+*/
+
 /*
  * This header defines short names for V8 constants for use by the ustack
  * helper.
  */
 
-#ifndef V8_ABBR_H
-#define V8_ABBR_H
+#ifndef SRC_V8ABBR_H_
+#define SRC_V8ABBR_H_
 
 /* Frame pointer offsets */
 #define V8_OFF_FP_FUNC              V8DBG_OFF_FP_FUNCTION
@@ -20,6 +42,7 @@
 #define V8_FT_INTERNAL              V8DBG_FRAMETYPE_INTERNALFRAME
 #define V8_FT_CONSTRUCT             V8DBG_FRAMETYPE_CONSTRUCTFRAME
 #define V8_FT_ADAPTOR               V8DBG_FRAMETYPE_ARGUMENTSADAPTORFRAME
+#define V8_FT_STUB                  V8DBG_FRAMETYPE_STUBFRAME
 
 /* Identification masks and tags */
 #define V8_SmiTagMask               (V8DBG_SMITAGMASK)
@@ -33,7 +56,7 @@
 #define V8_StringTag                V8DBG_STRINGTAG
 
 #define V8_StringEncodingMask       V8DBG_STRINGENCODINGMASK
-#define V8_AsciiStringTag           V8DBG_ASCIISTRINGTAG
+#define V8_AsciiStringTag           V8DBG_ONEBYTESTRINGTAG
 
 #define V8_StringRepresentationMask V8DBG_STRINGREPRESENTATIONMASK
 #define V8_SeqStringTag             V8DBG_SEQSTRINGTAG
@@ -50,15 +73,10 @@
 
 /*
  * Not all versions of V8 have the offset for the "chars" array in the
- * SeqTwoByteString class, but it's the same as the one for SeqOneByteString,
- * which used to be called SeqAsciiString.
+ * SeqTwoByteString class, but it's the same as the one for SeqOneByteString.
  */
 #ifndef V8DBG_CLASS_SEQTWOBYTESTRING__CHARS__CHAR
-#ifdef V8DBG_CLASS_SEQONEBYTESTRING__CHARS__CHAR
-#define	V8DBG_CLASS_SEQTWOBYTESTRING__CHARS__CHAR V8DBG_CLASS_SEQONEBYTESTRING__CHARS__CHAR
-#else
-#define	V8DBG_CLASS_SEQTWOBYTESTRING__CHARS__CHAR V8DBG_CLASS_SEQASCIISTRING__CHARS__CHAR
-#endif
+#define V8DBG_CLASS_SEQTWOBYTESTRING__CHARS__CHAR V8DBG_CLASS_SEQONEBYTESTRING__CHARS__CHAR
 #endif
 
 /* Heap class->field offsets */
@@ -81,7 +99,7 @@
 #define V8_OFF_STR_LENGTH \
     V8_OFF_HEAP(V8DBG_CLASS_STRING__LENGTH__SMI)
 #define V8_OFF_STR_CHARS  \
-    V8_OFF_HEAP(V8DBG_CLASS_SEQASCIISTRING__CHARS__CHAR)
+    V8_OFF_HEAP(V8DBG_CLASS_SEQONEBYTESTRING__CHARS__CHAR)
 #define V8_OFF_CONSSTR_CAR  \
     V8_OFF_HEAP(V8DBG_CLASS_CONSSTRING__FIRST__STRING)
 #define V8_OFF_CONSSTR_CDR  \
@@ -99,4 +117,4 @@
 #define V8_OFF_TWOBYTESTR_CHARS  \
     V8_OFF_HEAP(V8DBG_CLASS_SEQTWOBYTESTRING__CHARS__CHAR)
 
-#endif /* V8_ABBR_H */
+#endif  /* SRC_V8ABBR_H_ */
