@@ -5,8 +5,8 @@
 #include <v8.h>
 
 using node::AtExit;
-using v8::Handle;
 using v8::HandleScope;
+using v8::Isolate;
 using v8::Local;
 using v8::Object;
 
@@ -15,10 +15,10 @@ static int at_exit_cb1_called = 0;
 static int at_exit_cb2_called = 0;
 
 static void at_exit_cb1(void* arg) {
-  HandleScope scope;
-  assert(arg == 0);
-  Local<Object> obj = Object::New();
-  assert(!obj.IsEmpty()); // assert VM is still alive
+  Isolate* isolate = static_cast<Isolate*>(arg);
+  HandleScope handle_scope(isolate);
+  Local<Object> obj = Object::New(isolate);
+  assert(!obj.IsEmpty());  // Assert VM is still alive.
   assert(obj->IsObject());
   at_exit_cb1_called++;
 }
@@ -33,8 +33,8 @@ static void sanity_check(void) {
   assert(at_exit_cb2_called == 2);
 }
 
-void init(Handle<Object> target) {
-  AtExit(at_exit_cb1);
+void init(Local<Object> target) {
+  AtExit(at_exit_cb1, target->CreationContext()->GetIsolate());
   AtExit(at_exit_cb2, cookie);
   AtExit(at_exit_cb2, cookie);
   atexit(sanity_check);
