@@ -1,3 +1,5 @@
+'use strict';
+require('../../common');
 var assert = require('assert');
 var binding = require('./build/Release/binding');
 assert.equal('world', binding());
