@@ -1,3 +1,4 @@
+'use strict';
 // just like test/gc/http-client.js,
 // but aborting every connection that comes in.
 
@@ -11,37 +12,43 @@ var http  = require('http'),
     count   = 0,
     countGC = 0,
     todo    = 500,
-    common = require('../common.js'),
+    common = require('../common'),
     assert = require('assert'),
     PORT = common.PORT;
 
-console.log('We should do '+ todo +' requests');
+console.log('We should do ' + todo + ' requests');
 
 var http = require('http');
 var server = http.createServer(serverHandler);
 server.listen(PORT, getall);
 
 function getall() {
-  for (var i = 0; i < todo; i++) {
-    (function(){
-      function cb(res) {
-        done+=1;
-        statusLater();
-      }
+  if (count >= todo)
+    return;
 
-      var req = http.get({
-        hostname: 'localhost',
-        pathname: '/',
-        port: PORT
-      }, cb).on('error', cb);
+  (function() {
+    function cb(res) {
+      done += 1;
+      statusLater();
+    }
 
-      count++;
-      weak(req, afterGC);
-    })()
-  }
+    var req = http.get({
+      hostname: 'localhost',
+      pathname: '/',
+      port: PORT
+    }, cb).on('error', cb);
+
+    count++;
+    weak(req, afterGC);
+  })();
+
+  setImmediate(getall);
 }
 
-function afterGC(){
+for (var i = 0; i < 10; i++)
+  getall();
+
+function afterGC() {
   countGC ++;
 }
 
