@@ -1,9 +1,11 @@
+'use strict';
 // just like test/gc/http-client.js,
 // but with a timeout set
 
 function serverHandler(req, res) {
-  setTimeout(function () {
-    res.writeHead(200)
+  setTimeout(function() {
+    req.resume();
+    res.writeHead(200);
     res.end('hello\n');
   }, 100);
 }
@@ -14,41 +16,48 @@ var http  = require('http'),
     count   = 0,
     countGC = 0,
     todo    = 550,
-    common = require('../common.js'),
+    common = require('../common'),
     assert = require('assert'),
     PORT = common.PORT;
 
-console.log('We should do '+ todo +' requests');
+console.log('We should do ' + todo + ' requests');
 
 var http = require('http');
 var server = http.createServer(serverHandler);
 server.listen(PORT, getall);
 
 function getall() {
-  for (var i = 0; i < todo; i++) {
-    (function(){
-      function cb() {
-        done+=1;
-        statusLater();
-      }
+  if (count >= todo)
+    return;
 
-      var req = http.get({
-        hostname: 'localhost',
-        pathname: '/',
-        port: PORT
-      }, cb);
-      req.on('error', cb);
-      req.setTimeout(10, function(){
-        console.log('timeout (expected)')
-      });
+  (function() {
+    function cb(res) {
+      res.resume();
+      done += 1;
+      statusLater();
+    }
 
-      count++;
-      weak(req, afterGC);
-    })()
-  }
+    var req = http.get({
+      hostname: 'localhost',
+      pathname: '/',
+      port: PORT
+    }, cb);
+    req.on('error', cb);
+    req.setTimeout(10, function() {
+      console.log('timeout (expected)');
+    });
+
+    count++;
+    weak(req, afterGC);
+  })();
+
+  setImmediate(getall);
 }
 
-function afterGC(){
+for (var i = 0; i < 10; i++)
+  getall();
+
+function afterGC() {
   countGC ++;
 }
 
