@@ -1,38 +1,24 @@
-// Copyright Joyent, Inc. and other Node contributors.
-//
-// Permission is hereby granted, free of charge, to any person obtaining a
-// copy of this software and associated documentation files (the
-// "Software"), to deal in the Software without restriction, including
-// without limitation the rights to use, copy, modify, merge, publish,
-// distribute, sublicense, and/or sell copies of the Software, and to permit
-// persons to whom the Software is furnished to do so, subject to the
-// following conditions:
-//
-// The above copyright notice and this permission notice shall be included
-// in all copies or substantial portions of the Software.
-//
-// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
-// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
-// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
-// USE OR OTHER DEALINGS IN THE SOFTWARE.
-
+'use strict';
 var common = require('../common');
 var assert = require('assert');
 var spawn = require('child_process').spawn;
+
+if (!common.hasCrypto) {
+  console.log('1..0 # Skipped: missing crypto');
+  return;
+}
 var tls = require('tls');
+
 var fs = require('fs');
 
+if (!common.opensslCli) {
+  console.log('1..0 # Skipped: node compiled without OpenSSL CLI.');
+  return;
+}
+
 // renegotiation limits to test
 var LIMITS = [0, 1, 2, 3, 5, 10, 16];
 
-if (process.platform === 'win32') {
-  console.log('Skipping test, you probably don\'t have openssl installed.');
-  process.exit();
-}
-
 (function() {
   var n = 0;
   function next() {
@@ -63,10 +49,13 @@ function test(next) {
 
   server.listen(common.PORT, function() {
     var args = ('s_client -connect 127.0.0.1:' + common.PORT).split(' ');
-    var child = spawn('openssl', args);
+    var child = spawn(common.opensslCli, args);
 
-    child.stdout.pipe(process.stdout);
-    child.stderr.pipe(process.stderr);
+    //child.stdout.pipe(process.stdout);
+    //child.stderr.pipe(process.stderr);
+
+    child.stdout.resume();
+    child.stderr.resume();
 
     // count handshakes, start the attack after the initial handshake is done
     var handshakes = 0;
@@ -87,7 +76,14 @@ function test(next) {
 
     var closed = false;
     child.stdin.on('error', function(err) {
-      assert.equal(err.code, 'ECONNRESET');
+      switch (err.code) {
+        case 'ECONNRESET':
+        case 'EPIPE':
+          break;
+        default:
+          assert.equal(err.code, 'ECONNRESET');
+          break;
+      }
       closed = true;
     });
     child.stdin.on('close', function() {
